/**
 * File Name: PlayGame.js
 * File URL: http://www.renown-games.com/MozartMusicMatch/
 * Description: File for controlling and displaying game scenes; managing global variables throughout game state.
 * Author: Stephen Gose
 * Version: 0.0.0.15
 * Author URL: http://www.stephen-gose.com/
 * Support: support@pbmcube.com
 *
 * Copyright  2008-2016 Stephen Gose LLC. All rights reserved. 
 *
 * Redistribution of part or whole of this file and
 * the accompanying files is strictly prohibited.
 * 
 * Do not sell! Do not distribute!
 * This is a licensed permission file. Please refer to Terms of Use
 *  	and End Users License Agreement (EULA).
 * Search for [ //**TODO** ] to tailor this file for your own use and will void any support agreement.
 *
 * Transpiled using AS3 to JS - http://as3js.org/
 */
(function() {
  var Program = {};
  Program["PlayGame"] = function(module, exports) {
    module.inject = function() {
      PlayGame.boardWidth = 4;
      PlayGame.boardHeight = 4;
      PlayGame.cardHorizontalSpacing = 52;
      PlayGame.cardVerticalSpacing = 52;
      PlayGame.boardOffsetX = 300;
      PlayGame.boardOffsetY = 200;
      PlayGame.pointsForMatch = -10;
      PlayGame.pointsForMiss = 50;
    };

    var PlayGame = function(passed_class) {
      this.global_class = null;
      this.firstCard = null;
      this.secondCard = null;
      this.gameScoreField = null;
      this.gameTimeField = null;
      this.gameInstruction_txt = null;
      this.flipBackTimer = null;
      this.soundList = new Array;
      // constructor code
      this.global_class = passed_class;

      //reset this game timer and scores
      this.gameduration = getTimer();

      this.global_class.gameStartTime = this.gameduration;
      this.global_class.gameEndTime = 0;
      this.global_class.hints = 0;
      this.global_class.matches = 0;
      this.global_class.difficulty = 0;

      //navigation menu
      toGame_btn.addEventListener(MouseEvent.CLICK, this.on_click);

      // make a list of card numbers
      var cardlist = new Array;
      var soundPlayList = new Array;

      //==========================================================================
      // set up sounds - adopted from MemoryGame
      // load the sounds
      var soundList = new Array; // hold sounds
      for (var j = 1; j <= 8; j++) {
        var thisSound = new Sound;
        //this loads from my site; game is local to mp3 files
        var req = new URLRequest("http://cdn.pbmcube.com/winx/notepair_0" + j + ".mp3");
        thisSound.load(req);
        soundList.push(thisSound);
      }
      //==========================================================================

      for (var i = 0; i < PlayGame.boardWidth * PlayGame.boardHeight / 2; i++) {
        cardlist.push(i);
        cardlist.push(i);
        //new add sound with same index - toneNdx
        soundPlayList.push(i);
        soundPlayList.push(i);
      }
      // create all the cards, position them, and assign a randomcard face to each
      this.cardsLeft = 0;
      for (var x = 0; x < PlayGame.boardWidth; x++) { // horizontal
        for (var y = 0; y < PlayGame.boardHeight; y++) { // vertical
          var c = new Card10; // copy the movie clip
          c.stop(); // stop on first frame
          c.x = x * PlayGame.cardHorizontalSpacing + PlayGame.boardOffsetX; // set position
          c.y = y * PlayGame.cardVerticalSpacing + PlayGame.boardOffsetY;
          var r = Math.floor(Math.random() * cardlist.length); // get a random face
          c.cardface = cardlist[r]; // assign face to card
          c.toneNdx = soundPlayList[r];
          c.tone = soundList[c.toneNdx]; // assign sound to same card
          cardlist.splice(r, 1); // remove face from list
          soundPlayList.splice(r, 1); // remove sound from list
          c.addEventListener(MouseEvent.CLICK, this.clickCard); // have it listen for clicks
          c.buttonMode = true;
          addChild(c); // show the card
          this.cardsLeft++;
        }
      }
      // set up the score
      this.gameScoreField = new TextField;
      this.gameScoreField.x = 175;
      this.gameScoreField.y = 150;
      addChild(this.gameScoreField);
      this.gameScore = 0;
      this.showGameScore();

      // set up the clock
      this.gameTimeField = new TextField;
      this.gameTimeField.x = 172;
      this.gameTimeField.y = 140;
      addChild(this.gameTimeField);
      this.gameStartTime = getTimer();
      this.gameTime = 0;
      addEventListener(Event.ENTER_FRAME, this.showTime);
    };

    PlayGame.prototype = Object.create(Sprite.prototype);

    PlayGame.boardWidth = 4;
    PlayGame.boardHeight = 4;
    PlayGame.cardHorizontalSpacing = 52;
    PlayGame.cardVerticalSpacing = 52;
    PlayGame.boardOffsetX = 300;
    PlayGame.boardOffsetY = 200;
    PlayGame.pointsForMatch = -10;
    PlayGame.pointsForMiss = 50;

    PlayGame.prototype.global_class = null;
    PlayGame.prototype.firstCard = null;
    PlayGame.prototype.secondCard = null;
    PlayGame.prototype.cardsLeft = 0;
    PlayGame.prototype.gameScore = 0;
    PlayGame.prototype.gameStartTime = 0;
    PlayGame.prototype.gameTime = 0;
    PlayGame.prototype.gameScoreField = null;
    PlayGame.prototype.gameTimeField = null;
    PlayGame.prototype.gameInstruction_txt = null;
    PlayGame.prototype.flipBackTimer = null;
    PlayGame.prototype.soundList = null;
    PlayGame.prototype.gameduration = 0;
    PlayGame.prototype.clickCard = function(event) {
      var thisCard = event.target as Card10; // what card?

      if (this.firstCard == null) { // first card in a pair
        this.firstCard = thisCard; // note it
        thisCard.startFlip(thisCard.cardface + 2);
        //playSound(theFirstCardSound);
        //playSound(soundList(thisCard.toneNdx));
        this.playSound(this.firstCard.tone);

      } else if (this.firstCard == thisCard) { // clicked first card again
        this.firstCard.startFlip(1);
        this.firstCard = null;
        //playSound(theMissSound);

      } else if (this.secondCard == null) { // second card in a pair
        this.secondCard = thisCard; // note it
        thisCard.startFlip(thisCard.cardface + 2);
        this.playSound(this.secondCard.tone);

        // compare two cards
        if (this.firstCard.cardface == this.secondCard.cardface) {
          //playSound(firstCard.tone);
          //playSound(secondCard.tone);
          // remove a match
          removeChild(this.firstCard);
          removeChild(this.secondCard);
          // reset selection
          this.firstCard = null;
          this.secondCard = null;
          // add points
          this.gameScore += PlayGame.pointsForMatch;
          this.showGameScore();
          //playSound(theMatchSound);
          // check for game over
          this.cardsLeft -= 2; // 2 less cards
          //20151030 - New function to test for PuzzleDone
          this.pzlDone(this.cardsLeft);
          /**
          if (cardsLeft == 0) {
            MovieClip(root).gameScore=gameScore;
            MovieClip(root).gameTime=clockTime(gameTime);
            MovieClip(root).gotoAndStop("gameover");
          }
          */
        } else {
          this.gameScore += PlayGame.pointsForMiss;
          this.showGameScore();
          //playSound(secondCard.tone);
          //playSound(theMissSound);
          this.flipBackTimer = new Timer(1000, 1);
          this.flipBackTimer.addEventListener(TimerEvent.TIMER_COMPLETE, this.returnCards);
          this.flipBackTimer.start();
        }
      } else { // starting to pick another pair
        this.returnCards(null);
        //playSound(theFirstCardSound);

        // select first card in next pair
        this.firstCard = thisCard;
        this.firstCard.startFlip(thisCard.cardface + 2);
      }
    };
    PlayGame.prototype.returnCards = function(event) {
      this.firstCard.startFlip(1);
      this.secondCard.startFlip(1);
      this.playSound(this.firstCard.tone);
      this.playSound(this.secondCard.tone);

      this.firstCard = null;
      this.secondCard = null;
      this.flipBackTimer.removeEventListener(TimerEvent.TIMER_COMPLETE, this.returnCards);
    };
    PlayGame.prototype.showGameScore = function() {
      this.gameScoreField.text = "Score: " + String(this.gameScore);
    };
    PlayGame.prototype.showTime = function(event) {
      this.gameTime = getTimer() - this.gameStartTime;
      this.gameTimeField.text = "Time: " + this.clockTime(this.gameTime);
    };
    PlayGame.prototype.clockTime = function(ms) {
      var seconds = Math.floor(ms / 1000);
      var minutes = Math.floor(seconds / 60);
      seconds -= minutes * 60;
      var timeString = minutes + ":" + String(seconds + 100).substr(1, 2);
      return timeString;
    };
    PlayGame.prototype.playSound = function(soundObject) {
      var channel = soundObject.play();
    };
    PlayGame.prototype.GetTimeStamp = function() {
      var duration = getTimer();
      trace("duration: " + duration);
      return duration;
    };
    PlayGame.prototype.on_click = function(event) {
      this.global_class.gameState_MainMenu();
    };
    PlayGame.prototype.pzlDone = function(cardsLeft) {
      if (cardsLeft == 0) {
        //MovieClip(root).gameScore=gameScore;
        //MovieClip(root).gameTime=clockTime(gameTime);
        //MovieClip(root).gotoAndStop("gameover");

        //save finished puzzle to global variables
        this.global_class.matches = this.gameScore;
        //global_class.difficulty = Math.round(Math.sqrt(hRez*vRez))*2;
        this.global_class.gameEndTime = getTimer();
        //change game state to GameOver
        this.global_class.gameState_GameOver();
      } //PuzzleDone check

    }

    module.exports = PlayGame;
  };
  if (typeof module !== 'undefined') {
    module.exports = AS3JS.load({
      program: Program,
      entry: "PlayGame",
      entryMode: "instance"
    });
  } else if (typeof window !== 'undefined' && typeof AS3JS !== 'undefined') {
    window['PlayGame'] = AS3JS.load({
      program: Program,
      entry: "PlayGame",
      entryMode: "instance"
    });
  }
})();